local _G = getfenv(0)
local ADDON_NAME, addon = ...
local LibStub = _G.LibStub
local DKDiseases = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)
local L = LibStub("AceLocale-3.0"):GetLocale(addon.addonName)
local LSM = _G.LibStub:GetLibrary("LibSharedMedia-3.0")
local Masque = LibStub("Masque", true)

local SpellIds = addon.SpellIds
local SpellNames = addon.SpellNames
local SpellIcons = addon.SpellIcons
local FormatNumber = addon.FormatNumber

-- Locals for performance
local pairs = _G.pairs
local ipairs = _G.ipairs
local GetTime = _G.GetTime
local tostring = _G.tostring
local math = _G.math
local abs = _G.math.abs
local GetSpellCooldown = _G.GetSpellCooldown

local SRTracker = {}
addon:RegisterModule("SRTracker", SRTracker)
SRTracker.enabled = false
SRTracker.srPercent = 0.35
SRTracker.state = nil

local function EventFrame_OnEvent(frame, event, ...)
	if event == "UNIT_HEALTH" then
		SRTracker:UNIT_HEALTH(event, ...)
	elseif event == "UNIT_MAX_HEALTH" then
		SRTracker:UNIT_MAX_HEALTH(event, ...)
	elseif event == "SPELL_UPDATE_COOLDOWN" then
		SRTracker:SPELL_UPDATE_COOLDOWN(event, ...)
	end
end
local EventFrames = {}
local WatchedUnits = { "target" }
for i, unit in ipairs(WatchedUnits) do
	if not EventFrames[unit] then
		local frame = _G.CreateFrame("Frame", ADDON_NAME.."_EventFrame_SRTracker_"..unit)
		frame:SetScript("OnEvent", EventFrame_OnEvent)
		EventFrames[unit] = frame
	end
end

function SRTracker:RegisterCallbacks()
	if not addon.isDK then return end
	addon:RegisterCallback("TalentUpdate", "SRTracker", SRTracker.OnTalentUpdate)
	addon:RegisterCallback("StatUpdate", "SRTracker", SRTracker.OnStatUpdate)
	addon:RegisterCallback("PlayerAlive", "SRTracker", SRTracker.OnPlayerAlive)
	addon:RegisterCallback("PlayerDead", "SRTracker", SRTracker.OnPlayerDead)
	addon:RegisterCallback("TargetChanged", "SRTracker", SRTracker.OnTargetChanged)
	addon:RegisterCallback("Load", "SRTracker", SRTracker.OnLoad)
end

function SRTracker:SetProfile()
	self.profile = DKDiseases.db.profile.srFrame
end

function SRTracker:OnProfileUpdate()
	SRTracker:SetProfile()
end

function SRTracker:OnCombatStart()
	SRTracker:SetAlpha()
end

function SRTracker:OnCombatEnd()
	SRTracker:SetAlpha()
end

function SRTracker:OnInitialize()
	self:SetProfile()
	addon:RegisterCallback("ProfileUpdate", "SRTracker", SRTracker.OnProfileUpdate)
	self:RegisterCallbacks()
end

function SRTracker:Enable()
	self:RegisterCallbacks()
end

function SRTracker:Disable()
	addon:UnregisterCallback("TalentUpdate", "SRTracker")
	addon:UnregisterCallback("StatUpdate", "SRTracker")
	addon:UnregisterCallback("PlayerAlive", "SRTracker")
	addon:UnregisterCallback("PlayerDead", "SRTracker")
	addon:UnregisterCallback("TargetChanged", "SRTracker")
	addon:UnregisterCallback("Load", "SRTracker")
	addon:UnregisterCallback("CombatStart", "SRTracker")
	addon:UnregisterCallback("CombatEnd", "SRTracker")
end

function SRTracker:OnTalentUpdate()
	SRTracker.srPercent = addon.currentSpec == "Unholy" and 0.45 or 0.35
	if addon.srFrame then addon.srFrame:Update() end
	-- Reset the icon used to Soul Reaper.
	addon.srFrame.texture:SetTexture(SpellIcons["Soul Reaper"])
	addon.srFrame.showing = "SR"
	SRTracker:UpdateSoulReaperCheck()
end

function SRTracker:OnPlayerAlive()
	if addon.srFrame and not addon.db.profile.srFrame.alwaysShow then
		addon.srFrame:Hide()
	end
end

function SRTracker:OnPlayerDead()
	if addon.srFrame and not addon.db.profile.srFrame.alwaysShow then
		addon.srFrame:Hide()
	end
end

function SRTracker:OnTargetChanged()
	addon.soulReaperUsable = nil
	addon.targetMaxHealth = _G.UnitHealthMax("target") or 0
	local db = addon.db.profile.srFrame
	if db.enabled and not db.alwaysShow then
		SRTracker:CheckSoulReaper()
	end
end

function SRTracker:OnLoad()
	SRTracker:CreateSoulReaperCheck()
	SRTracker:UpdateSoulReaperCheck()
end

function SRTracker:CreateSoulReaperCheck()
	if addon.srFrame then return end

	local buttonName = "DKDiseases_SRCheck"
	local frame = _G.CreateFrame("Frame", "DKDiseases_SRCheck", _G.UIParent)
	addon.srFrame = frame

	frame.SetDesiredPoint = function(frame)
		frame:ClearAllPoints()
		local profile = addon.db.profile.srFrame
		local x, y = profile.x, profile.y
    	frame:SetPoint("CENTER", _G.UIParent, "CENTER", x, y)
	end
	frame:SetDesiredPoint()
	frame.SetDimensions = function(frame)
	    frame:SetWidth(addon.db.profile.srFrame.width)
	    frame:SetHeight(addon.db.profile.srFrame.height)
	end
	frame:SetDimensions()

	frame.texture = frame:CreateTexture(nil, "ARTWORK")
    frame.texture:SetAllPoints(frame)
    frame.texture:SetVertexColor(1, 1, 1)
	if SpellIcons["Soul Reaper"] then
		frame.texture:SetTexture(SpellIcons["Soul Reaper"])
	end
	frame.showing = nil

	frame.border = frame:CreateTexture(nil, "BACKGROUND")
	frame.border:SetPoint("CENTER")
	frame.border:SetWidth(frame:GetWidth())
	frame.border:SetHeight(frame:GetHeight())

	frame.value = frame:CreateFontString(nil, "OVERLAY")
	frame.value:SetPoint("TOP", frame, "BOTTOM", 0, -4)
	local ff, fh, fflags = DKDiseases:GetFontSettings()
	local font = LSM:Fetch("font", ff)
	frame.value:SetFont(font, fh, fflags)
	frame.value:SetJustifyH("CENTER")
	frame.value:SetJustifyV("CENTER")

	frame.ResetFonts = function()
		local ff, fh, fontFlags = DKDiseases:GetFontSettings()
		local font = LSM:Fetch("font", ff)
		frame.value:SetFont(font, fh, fontFlags)						
		frame.value:SetText(frame.value:GetText())
	end

	frame.Update = function(self)
		local profile = addon.db.profile.srFrame
		SRTracker:UpdateSoulReaperCheck()
		if profile.showText then
			self.value:Show()
		else
			self.value:Hide()
		end
	end

	frame.locked = true
	frame.Lock = function(self, locked)
		if locked ~= nil then
			self.locked = locked
		end
		self:EnableMouse(not self.locked)
	end
	frame.IsLocked = function(self)
		return self.locked
	end

    frame:SetMovable(true)
    frame:RegisterForDrag("LeftButton")
    frame:SetScript("OnDragStart",
        function(self, button)
			if not self.locked then
            	self:StartMoving()
			end
        end)
    frame:SetScript("OnDragStop",
        function(self)
            self:StopMovingOrSizing()
			local scale = self:GetEffectiveScale() / _G.UIParent:GetEffectiveScale()
			local x, y = self:GetCenter()
			x, y = x * scale, y * scale
			x = x - _G.GetScreenWidth()/2
			y = y - _G.GetScreenHeight()/2
			x = x / self:GetScale()
			y = y / self:GetScale()
			local profile = addon.db.profile.srFrame
			profile.x, profile.y = x, y
			profile.userPlaced = true
			self:SetUserPlaced(false)
        end)
	frame:Lock()

	if Masque then
		addon.MasqueGroup = Masque:Group("DKDiseases", SpellNames["Soul Reaper"])
		frame.MasqueButtonData = {
			--Button = frame
			Icon = frame.texture,
			FloatingBG = false,
			Cooldown = false,
			Flash = false,
			Pushed = false,
			Normal = false,
			Disabled = false,
			Checked = false,
			Border = frame.border,
			AutoCastable = false,
			Highlight = false,
			HotKey = false,
			Count = false,
			Name = false,
			Duration = false,
			AutoCast = false,
		}
		addon.MasqueGroup:AddButton(frame, frame.MasqueButtonData)
	end
end

function SRTracker:ToggleTracking(enable)
	local eventFrame = EventFrames["target"]
	if enable then
		if eventFrame then
			eventFrame:RegisterEvent("SPELL_UPDATE_COOLDOWN")
			eventFrame:RegisterUnitEvent("UNIT_HEALTH", "target")
			eventFrame:RegisterUnitEvent("UNIT_MAX_HEALTH", "target")
			addon:RegisterCallback("CombatStart", "SRTracker", SRTracker.OnCombatStart)
			addon:RegisterCallback("CombatEnd", "SRTracker", SRTracker.OnCombatEnd)
		end
	else
		if eventFrame then
			eventFrame:UnregisterEvent("SPELL_UPDATE_COOLDOWN")
			eventFrame:UnregisterEvent("UNIT_HEALTH")
			eventFrame:UnregisterEvent("UNIT_MAX_HEALTH")
			addon:UnregisterCallback("CombatStart", "SRTracker")
			addon:UnregisterCallback("CombatEnd", "SRTracker")
		end
	end
end

function SRTracker:UpdateSoulReaperCheck()
	if not addon.isDK then return end

	self.soulReaperId = addon.soulReaperIds[addon.currentSpec or ""] or 
		SpellIds["Soul Reaper (Unholy)"]

	local eventFrame = EventFrames["target"]
	if addon.db.profile.srFrame.enabled and addon.currentSpec and
		addon.db.profile.srFrame.specs[addon.currentSpec] and
		_G.IsSpellKnown(SpellIds["Soul Reaper (Unholy)"]) then
		self.active = true
		if addon.db.profile.srFrame.alwaysShow and addon.currentSpec == "Frost" then
			self:ToggleTracking(false)
			if addon.srFrame then
				self:SetAlpha("always")
				addon.srFrame:Show()
			end
		else
			self:ToggleTracking(true)
			if addon.srFrame then
				self:SetAlpha()
				addon.srFrame:Show()
			end
		end
	else
		self.active = false
		self:ToggleTracking(false)
		if addon.srFrame then addon.srFrame:Hide() end
	end
end

function SRTracker:CalculateSoulReaper()
	if addon.currentSpec == "Frost" then
		-- Calculate use of SR vs HB
		addon.soulReaper1 = addon.avgWeaponDmg * 0.85
		addon.soulReaper2 = addon.soulReaperBase + (3.337 * addon.effectiveAP)
		addon.howlingBlast = (499 + 0.90 * addon.effectiveAP) * 
			(1 + (2.0 * addon.mastery / 100)) * addon.frostVulnerability
		local fmt = "%s - %s"
		if addon.srFrame and addon.db.profile.srFrame.enabled then
			addon.srFrame.value:SetText(fmt:format(
				FormatNumber(addon.soulReaper1 + addon.soulReaper2),
				FormatNumber(addon.howlingBlast)))
			local useSpell = "SR"
			if addon.howlingBlast * addon.spellCrit > 
				(addon.soulReaper1 * addon.critChance + 
				 addon.soulReaper2 * addon.spellCrit) then
				useSpell = "HB"
			end
			if useSpell ~= addon.srFrame.showing then
				addon.srFrame.texture:SetTexture(
					useSpell == "SR" and SpellIcons["Soul Reaper"] or 
						SpellIcons["Howling Blast"])
				addon.srFrame.showing = useSpell
			end
		end
	end
end

function SRTracker:CheckSoulReaper()
	local profile = addon.db.profile.srFrame
	if not addon.srFrame or not profile.enabled or not self.active then
		if addon.srFrame then addon.srFrame:Hide() end
		return
	end
	if addon.hostile.target and not _G.UnitIsDeadOrGhost("player") then
		local health = _G.UnitHealth("target") or 0
		local percent = addon.targetMaxHealth > 0 and 
			health / addon.targetMaxHealth or 0
		local srRange = percent < profile.percent and health > 0
		if addon.db.profile.srFrame.onlyShowHB and addon.currentSpec == "Frost" then
			if srRange and addon.srFrame.showing == "HB" then
				addon.srFrame:Show()
			else
				addon.srFrame:Hide()
			end
		elseif srRange and not addon.srActive then
			self:SetAlpha("usable")
		else
			self:SetAlpha("unusable")
		end
		--addon.soulReaperUsable = srRange
	else
		self:SetAlpha("ooc")
	end
end

function SRTracker:SetAlpha(setting, force)
	local alpha

	if addon.db.profile.srFrame.onlyShowHB and addon.currentSpec == "Frost" then
		setting = "always"
	elseif not _G.UnitAffectingCombat("player") then
		setting = "ooc"
	end

	if force or setting ~= self.state then
		if setting then
			if setting == "ooc" then
				alpha = self.profile.alphas.ooc
			elseif setting == "usable" or setting == "always" then
				alpha = self.profile.alphas.usable
			elseif setting == "unusable" then
				alpha = self.profile.alphas.unusable
			elseif setting == "cd" then
				alpha = self.profile.alphas.cooldown
			end
		end
	end
	if setting and alpha then
		self.state = setting
		addon.srFrame:SetAlpha(alpha)
	end
end

function SRTracker:SPELL_UPDATE_COOLDOWN(event)
	local srActive = false
	local start, duration, enabled = GetSpellCooldown(self.soulReaperId)
	if duration >= 10 and self.profile.ignoreRuneCD then
		srActive = addon.srActive
	elseif duration > 0 then
		local timeLeft = duration - (GetTime() - start)
		if timeLeft >= 1 then srActive = true end
	end
	if srActive ~= addon.srActive then
		addon.srActive = srActive 
		self:CheckSoulReaper()
	end
end

function SRTracker:UNIT_HEALTH(event, unit)
	if unit == "target" then
		self:CheckSoulReaper()
	end
end

function SRTracker:UNIT_MAX_HEALTH(event, unit)
	if unit == "target" then
		addon.targetMaxHealth = _G.UnitHealthMax("target") or 0
		self:CheckSoulReaper()
	end
end

function SRTracker:GetOptions()
	return "SRTracker", SpellNames["Soul Reaper"], self:GetModuleOptions()
end

function SRTracker:GetModuleOptions()
	local srTrackerOpts = {
		generalOptions = {
			order = 2,
			type = "header",
			name = L["General Options"],
		},
		srEnabled = {
			name = L["Enabled"],
			desc = L["Enabled"],
			type = "toggle",
			order = 10,
			set = function(info, val)
				addon.db.profile.srFrame.enabled = val
				--DKDiseases:ToggleStatTracking()
				if addon.srFrame then addon.srFrame:Update() end
			end,
			get = function(info)
				return addon.db.profile.srFrame.enabled
			end,
		},
		showValue = {
			name = L["Value"],
			desc = L["Value"],
			type = "toggle",
			order = 20,
			set = function(info, val)
				addon.db.profile.srFrame.showText = val
				if addon.srFrame then addon.srFrame:Update() end
			end,
			get = function(info)
				return addon.db.profile.srFrame.showText
			end,
		},
		specHdr = {
			order = 110,
			type = "header",
			name = L["Enabled for Specs"],
		},
		specBlood = {
			name = L["Blood"],
			desc = L["Blood"],
			type = "toggle",
			order = 111,
			set = function(info, val)
				addon.db.profile.srFrame.specs["Blood"] = val
				self:UpdateSoulReaperCheck()
			end,
			get = function(info)
				return addon.db.profile.srFrame.specs["Blood"]
			end,
		},
		specFrost = {
			name = L["Frost"],
			desc = L["Frost"],
			type = "toggle",
			order = 112,
			set = function(info, val)
				addon.db.profile.srFrame.specs["Frost"] = val
				self:UpdateSoulReaperCheck()
			end,
			get = function(info)
				return addon.db.profile.srFrame.specs["Frost"]
			end,
		},
		specUnholy = {
			name = L["Unholy"],
			desc = L["Unholy"],
			type = "toggle",
			order = 113,
			set = function(info, val)
				addon.db.profile.srFrame.specs["Unholy"] = val
				self:UpdateSoulReaperCheck()
			end,
			get = function(info)
				return addon.db.profile.srFrame.specs["Unholy"]
			end,
		},
		percentHdr = {
			order = 130,
			type = "header",
			name = L["Percent"],
		},
		srPercentAuto = {
			name = L["Override"],
			desc = L["Override"],
			type = "toggle",
			order = 131,
			set = function(info, val)
				addon.db.profile.srFrame.percentOverride = val
			end,
			get = function(info)
				return addon.db.profile.srFrame.percentOverride
			end,
		},
		srPercent = {
			order = 132,
			name = L["Percent"],
			desc = L["Percent"],
			type = "range",
			min = 0.01,
			max = 1.0,
			step = 0.01,
			bigStep = 0.05,
			isPercent = true,
			set = function(info, val)
				addon.db.profile.srFrame.percent = val
			end,
			get = function(info, val)
				return addon.db.profile.srFrame.percent
			end,
			disabled = function()
				return not addon.db.profile.srFrame.percentOverride
			end,
		},
		frostHdr = {
			order = 200,
			type = "header",
			name = L["Frost"],
		},
		alwaysShow = {
			name = L["Always Show"],
			desc = L["Always Show"],
			type = "toggle",
			order = 210,
			set = function(info, val)
				addon.db.profile.srFrame.alwaysShow = val
				addon.soulReaperUsable = nil
				if addon.srFrame then
					if val then addon.srFrame:Show() else addon.srFrame:Hide() end
					self:UpdateSoulReaperCheck()
				end
			end,
			get = function(info)
				return addon.db.profile.srFrame.alwaysShow
			end,
		},
		onlyShowHB = {
			name = L["Only Show HB"],
			desc = L["OnlyShowHB_Desc"],
			type = "toggle",
			order = 220,
			set = function(info, val)
			    addon.db.profile.srFrame.onlyShowHB = val
			end,
			get = function(info)
				return addon.db.profile.srFrame.onlyShowHB
			end,
		},
		dimensionsHdr = {
			order = 300,
			type = "header",
			name = L["Dimensions"],
		},
		width = {
			order = 310,
			name = L["Width"],
			desc = L["Width"],	
			type = "range",
			min = 20,
			max = 500,
			step = 1,
			set = function(info, val)
				addon.db.profile.srFrame.width = val
				if addon.srFrame then addon.srFrame:SetDimensions() end
			end,
			get = function(info, val)
				return addon.db.profile.srFrame.width
			end,
		},
		height = {
			order = 320,
			name = L["Height"],
			desc = L["Height"],	
			type = "range",
			min = 20,
			max = 500,
			step = 1,
			set = function(info, val)
				addon.db.profile.srFrame.height = val
				if addon.srFrame then addon.srFrame:SetDimensions() end
			end,
			get = function(info, val)
				return addon.db.profile.srFrame.height
			end,
		},
		positionHdr = {
			order = 400,
			type = "header",
			name = L["Position"],
		},
		x = {
			order = 410,
			name = L["X Offset"],
			desc = L["XOffset_Desc"],	
			type = "range",
			softMin = -math.floor(_G.GetScreenWidth()),
			softMax = math.floor(_G.GetScreenWidth()),
			bigStep = 1,
			set = function(info, val)
				addon.db.profile.srFrame.x = val
				if addon.srFrame then addon.srFrame:SetDesiredPoint() end
			end,
			get = function(info, val)
				return addon.db.profile.srFrame.x
			end,
		},
		y = {
			order = 420,
			name = L["Y Offset"],
			desc = L["YOffset_Desc"],	
			type = "range",
			softMin = -math.floor(_G.GetScreenHeight()),
			softMax = math.floor(_G.GetScreenHeight()),
			bigStep = 1,
			set = function(info, val)
				addon.db.profile.srFrame.y = val
				if addon.srFrame then addon.srFrame:SetDesiredPoint() end
			end,
			get = function(info, val)
				return addon.db.profile.srFrame.y
			end,
		},
	}
	return srTrackerOpts
end
